aboutsummaryrefslogtreecommitdiffhomepage
path: root/lib/messages.c
diff options
context:
space:
mode:
Diffstat (limited to 'lib/messages.c')
-rw-r--r--lib/messages.c80
1 files changed, 52 insertions, 28 deletions
diff --git a/lib/messages.c b/lib/messages.c
index 8b627750..a588f8f8 100644
--- a/lib/messages.c
+++ b/lib/messages.c
@@ -22,72 +22,96 @@
#include <glib.h> /* GList */
-typedef struct _message_list {
- notmuch_message_t *message;
- struct _message_list *next;
-} message_list_t;
struct _notmuch_messages {
- message_list_t *head;
- message_list_t **tail;
+ notmuch_message_node_t *iterator;
};
/* Create a new notmuch_messages_t object, with 'ctx' as its talloc owner.
*
* This function can return NULL in case of out-of-memory.
*/
-notmuch_messages_t *
-_notmuch_messages_create (void *ctx)
+notmuch_message_list_t *
+_notmuch_message_list_create (const void *ctx)
{
- notmuch_messages_t *messages;
+ notmuch_message_list_t *list;
- messages = talloc (ctx, notmuch_messages_t);
- if (unlikely (messages == NULL))
+ list = talloc (ctx, notmuch_message_list_t);
+ if (unlikely (list == NULL))
return NULL;
- messages->head = NULL;
- messages->tail = &messages->head;
+ list->head = NULL;
+ list->tail = &list->head;
- return messages;
+ return list;
}
-/* Add a new message to 'messages'.
+/* Append 'node' (which can of course point to an aribtrarily long
+ * list of nodes) to the end of 'list'.
*/
void
-_notmuch_messages_add_message (notmuch_messages_t *messages,
- notmuch_message_t *message)
+_notmuch_message_list_append (notmuch_message_list_t *list,
+ notmuch_message_node_t *node)
+{
+ *(list->tail) = node;
+ list->tail = &node->next;
+}
+
+/* Allocate a new node for 'message' and append it to the end of
+ * 'list'.
+ */
+void
+_notmuch_message_list_add_message (notmuch_message_list_t *list,
+ notmuch_message_t *message)
+{
+ notmuch_message_node_t *node = talloc (list, notmuch_message_node_t);
+
+ node->message = message;
+ node->next = NULL;
+
+ _notmuch_message_list_append (list, node);
+}
+
+notmuch_messages_t *
+_notmuch_messages_create (notmuch_message_list_t *list)
{
- message_list_t *new = talloc (messages, message_list_t);
+ notmuch_messages_t *messages;
- new->message = message;
- new->next = NULL;
+ if (list->head == NULL)
+ return NULL;
- *(messages->tail) = new;
- messages->tail = &new->next;
+ messages = talloc (list, notmuch_messages_t);
+ if (unlikely (messages == NULL))
+ return NULL;
+
+ messages->iterator = list->head;
+
+ return messages;
}
notmuch_bool_t
notmuch_messages_has_more (notmuch_messages_t *messages)
{
- return messages->head != NULL;
+ return (messages != NULL && messages->iterator != NULL);
}
notmuch_message_t *
notmuch_messages_get (notmuch_messages_t *messages)
{
- if (messages->head == NULL)
+ if (messages->iterator == NULL)
return NULL;
- return messages->head->message;
+ return messages->iterator->message;
}
void
notmuch_messages_advance (notmuch_messages_t *messages)
{
- if (messages->head == NULL)
+ if (messages->iterator == NULL)
return;
- messages->head = messages->head->next;
+ messages->iterator = messages->iterator->next;
}
void
@@ -22,72 +22,96 @@
};
-/* Create a new notmuch_messages_t object, with 'ctx' as its talloc owner.
+/* Create a new notmuch_message_list_t object, with 'ctx' as its
+ * talloc owner.
*
* This function can return NULL in case of out-of-memory.